#!/usr/bin/env python

import time
import json
from pyueye import ueye
import threading

class SLMCapture:

	# initializing of the IDS camera, all parameters taken from the main
	def __init__(self, hCam, cInfo, pcImageMemory, MemID, rectAOI, pitch, nBitsPerPixel, m_nColorMode, bytes_per_pixel):
		
		self.hCam = hCam
		self.cInfo = cInfo
		self.pcImageMemory = pcImageMemory
		self.MemID = MemID
		self.rectAOI = rectAOI
		self.pitch = pitch
		self.nBitsPerPixel = nBitsPerPixel
		self.m_nColorMode = m_nColorMode
		self.bytes_per_pixel = bytes_per_pixel
		self.width = 0								# here initialized, later defined
		self.height = 0								# here initialized, later defined
		
	def Initialization(self):
		
		nRet = ueye.is_InitCamera(self.hCam, None)
		allright0 = 1																				# start drivers, connect to camera
		if nRet != ueye.IS_SUCCESS:
			print("is_InitCamera ERROR")
			allright0 = 0

		nRet = ueye.is_GetCameraInfo(self.hCam, self.cInfo)
		allright1 = 1																		# read camera memory data and give it to where cInfo points
		if nRet != ueye.IS_SUCCESS:
			print("is_GetCameraInfo ERROR")
			allright1 = 0

		nRet = ueye.is_AOI(self.hCam, ueye.IS_AOI_IMAGE_GET_AOI, self.rectAOI, ueye.sizeof(self.rectAOI))						# AOI setting
		allright2 = 1
		if nRet != ueye.IS_SUCCESS:
			print("is_AOI ERROR")
			allright2 = 0

		self.width = self.rectAOI.s32Width
		self.height = self.rectAOI.s32Height

		return(self.width,self.height,allright0,allright1,allright2)
		
	def Memalloc(self):
	
		nRet = ueye.is_AllocImageMem(self.hCam, self.width, self.height, self.nBitsPerPixel, self.pcImageMemory, self.MemID)	# allocate image memory to pcImageMemory
		if nRet != ueye.IS_SUCCESS:
			print("is_AllocImageMem ERROR")
			return (0)
		else:
			nRet = ueye.is_SetImageMem(self.hCam, self.pcImageMemory, self.MemID)												# makes the specified image memory the active memory
			if nRet != ueye.IS_SUCCESS:
				print("is_SetImageMem ERROR")
				return (0)
			else:
				nRet = ueye.is_SetColorMode(self.hCam, self.m_nColorMode)
		return 1														# set the desired color mode

	def Queue(self):
	
		nRet = ueye.is_InquireImageMem(self.hCam, self.pcImageMemory, self.MemID, self.width, self.height, self.nBitsPerPixel, self.pitch)
		if nRet != ueye.IS_SUCCESS:
			print("is_InquireImageMem ERROR")
			return (0)
		return 1

	def Acquisition(self):

		nRet = ueye.is_FreezeVideo(self.hCam, ueye.IS_DONT_WAIT)
			
		if nRet != ueye.IS_SUCCESS:
			print("is_CaptureImage ERROR")

		time.sleep(0.2)																											# sleep time to allow the camera to correctly store the image in its memory
		array = ueye.get_data(self.pcImageMemory, self.width, self.height, self.nBitsPerPixel, self.pitch, copy=False)			# get the data from the camera
		return (array,nRet)
		
	def Exit(self):

		ueye.is_FreeImageMem(self.hCam, self.pcImageMemory, self.MemID)
		ueye.is_ExitCamera(self.hCam)