#!/usr/bin/env python

import time
import datetime
from pyueye import ueye
import sys
from SystemManager import *
from SLMCapture import *
from CircuitSet import *
from LOGManager import *
from SystemThreads import *

print("-----------------------------")
print("START  :  " + str(datetime.datetime.now()))
print("-----------------------------")

## Variables
hCam = ueye.HIDS(0)             			# 0: first available camera;  1-254: The camera with the specified camera ID -> visible/settable through software "IDS camera manager"
cInfo = ueye.CAMINFO()
pcImageMemory = ueye.c_mem_p()
MemID = ueye.int()
rectAOI = ueye.IS_RECT()				# default rectangular dimension
pitch = ueye.INT()
nBitsPerPixel = ueye.INT(8)
m_nColorMode = ueye.IS_CM_MONO8
bytes_per_pixel = int(nBitsPerPixel / 8)	

trighigh = '0'
triglow = '2'
auth = 0
w = 0							# counter
errcounter = 0						# counter of missed acquisition, initialize the camera all over if it reaches a threshold
startcounter = 0					# counter for robustness of start trigger (to avoid fake negative)
missed = []
port = '/dev/ttyACM0'
baud = 2000000

# create the objects
Inspector = LOGManager()						# handler of LOG
Butler = CircuitSet(port,baud)
Doc = SystemManager(Inspector)					# manage the system operations, such as files and fodlers creation, and reboot and poweroff
Eve = SLMCapture(hCam,cInfo,pcImageMemory,MemID,rectAOI,pitch,nBitsPerPixel,m_nColorMode,bytes_per_pixel)			# manage the camera
Chronos = TimerThread()							# chronometer, used to measure timeout
allright = [1,1,1,1,1]					# error array

## Camera initialization, return frame dimensions and error flags
(width,height,allright[0],allright[1],allright[2]) = Eve.Initialization()

## Starting of the threads
Chronos.start()

# while not (Butler.Start()):	# uncomment and remove below once the starting trigger is in place
#	pass
#Butler.Trigger(button,0)

time.sleep(1)

layers = Doc.parameters()		# this will be erased once the start is done by trigger
wmax = int(layers)*2 + 1		# this will be erased once the start is done by trigger

## Create the folder with date and time, for multiple acquisition. Takes in input the name of the external memory, return the path
tag = Doc.folder()

Pablo = ExportThread(Inspector,tag,height,width,bytes_per_pixel)				# exporter of array to memory
Pablo.start()

allright[3] = Eve.Memalloc()
allright[4] = Eve.Queue()

## Update the LOG with the informations currently available
Inspector.register(tag)
Inspector.camera(allright)

## Error check in camera operation. A "0" in the array would mean the camera cannot properly work, thus going for a reboot
if 0 in allright:
	print("Fatal error in camera initialization")
	Inspector.write()
	time.sleep(5)
	quit()
	
print("System initialized")
# second while cycle, the script perform the acquisitions and convert the data in images

while True:

	#if Butler.Start():
	startcounter = 0				############################################### START INDENT

	Chronos.enable(600)
	Butler.Trigger(trighigh,1)
	delta = Chronos.safe()

	if delta == 1:
		print("timeout expired  :  " + str(datetime.datetime.now()))
		time.sleep(3)
		break

	Butler.Trigger(triglow,0)

	(array,flag1) = Eve.Acquisition()	# returns the array and a flag that signal if the acquisition was a success

	if (flag1 != ueye.IS_SUCCESS):
		missed.append(w)		        # notes the missing acquisition to avoid a stop in the conversion
		Inspector.acquisition(w)	    # notes the error in the LOG
		print("failed acquisition " + str(w) + "  :  " + str(datetime.datetime.now()))
		errcounter = errcounter + 1
	else:
		errcounter = 0			        # the error counter is a safety measure against the stop of the camera, if it acquire again a frame the counter reset

	array1 = array.copy()			    # NEEDED, or it associate the camera memory with the result of the function, thus overwriting it at the next acquisition

	Pablo.newby(flag1, w, array1)		# a new array goes to export, 

	w = w + 1
	if (w == wmax-1):
		break

	Butler.Trigger(trighigh,0)

	#Butler.Trigger(triglow,0)
	time.sleep(3.6)
	(array,flag2) = Eve.Acquisition()

	if (flag2 != ueye.IS_SUCCESS):
		missed.append(w)
		Inspector.acquisition(w)
		print("failed acquisition " + str(w) + "  :  " + str(datetime.datetime.now()))
		errcounter = errcounter + 1
	else:
		errcounter = 0

	array2 = array.copy()

	Pablo.newby(flag2, w, array2)

	w = w + 1
	if (w == wmax-1):
		break

	if errcounter >= 20:		# if there are more than tot failed acquisition in a row, the camera memory gets released and the initialization is repeated
		Eve.exit()
		time.sleep(2)
		(width,height,allright[0],allright[1],allright[2]) = Eve.Initialization()
		allright[3] = Eve.Memalloc()
		allright[4] = Eve.Queue()
		if 0 in allright:
			print("camera disconnected  :  " + str(datetime.datetime.now()))
			Doc.log()
			quit()									################################################################ STOP INDENT
	#else:
	#	startcounter = startcounter + 1
	#	if startcounter == 5:
	#		break
			
# post acquisition, for comparison

print("out of the cycle")

time.sleep(5)
(array_post,flag_post) = Eve.Acquisition()

if (flag_post != ueye.IS_SUCCESS):
	missed.append(w)
	Inspector.acquisition(w)
	print("failed post_acquisition " + str(w) + "  :  " + str(datetime.datetime.now()))


Pablo.newby(flag_post, w, array_post)

while auth == 0:
	auth = Pablo.authorization()

Eve.Exit()

Chronos.stop()
Pablo.stop()

Inspector.write()
print("-----------------------------")
print("END  :  " + str(datetime.datetime.now()))
print("-----------------------------")
#Doc.reboot()
