import threading
import time
import RPi.GPIO as GPIO

class CircuitSet:
	
	# initializing the pins that recieve the various input
	def __init__(self, button, start, trigger):

		self.buttonpin = button
		self.trigpin = trigger
		self.startpin = start

		GPIO.setwarnings(False)
		GPIO.setmode(GPIO.BOARD)													# refers to the board's pins and not the chip's
		GPIO.setup(self.buttonpin, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)				# set the button pin as input, low
		GPIO.setup(self.startpin, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)				# set the start pin as input, low
		GPIO.setup(self.trigpin, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)				# set the trigger pin as input, low

	# Get the input pin value
	def Read(self, pin):

		return GPIO.input(pin)

	# Keep the program idle until a rising edge occurs
	def Trigger(self, pin, timeout):

		if timeout == 1:															# if the option timeout is 1, the method will stop waiting after the decided amount of time
			GPIO.wait_for_edge(pin, GPIO.RISING, timeout=700000)
		else:
			GPIO.wait_for_edge(pin, GPIO.RISING)

	# same as 'Trigger', this method waits for the falling edge, without the timeout option, not interesting for our purposes
	def Falling(self, pin):
		
		GPIO.wait_for_edge(pin, GPIO.FALLING)

	# reset the GPIO initial condition
	def CircuitCleanup(self):
		
		GPIO.cleanup()													# set GPIO as default, good practice


# class meant to handle GPIO in a thread (in particular to signal through LED without interrupting the code from running)
class BlinkThread(threading.Thread):

	def __init__(self, LED, interval):
		
		threading.Thread.__init__(self)
		self.LED = LED					# pin
		self.interval = interval		# default semiperiod for blinking
		self.flag = 0
		self.kill = 0
		GPIO.setup(LED, GPIO.OUT)
		GPIO.output(LED, 0)

	# start the LED blinking
	def enable(self):

		self.flag = 1

	# set the frequency for the blinking (namely, the semiperiod)
	def delay(self,time):
		
		self.interval = time

	# stop the blinking, putting the thread on hold
	def pause(self):

		self.flag = 0

	# stop the thread
	def stop(self):

		self.kill = 1

	# set the output to 1
	def steadyON(self):

		GPIO.output(self.LED,1)
	
	# set the output to 0
	def steadyOFF(self):

		GPIO.output(self.LED,0)

	# blink until 'stop' is called, and while the flag is 1
	def run(self):
		
		while (self.kill == 0):
			if self.flag == 1:
				GPIO.output(self.LED, 1)
				time.sleep(self.interval)
				GPIO.output(self.LED, 0)
				time.sleep(self.interval)