import datetime
import time
import os
import json
import threading

class SystemManager:
	
	def __init__(self, Alarm, Inspector):
		
		self.Alarm = Alarm				# LED handler, any error will be signaled with the red LED
		self.Inspector = Inspector		# LOG handler, any operation will be 
		self.path = ""
		
	# shut down the system
	def poweroff(self):
		
		os.system("sudo poweroff")
	
	# reboot the system
	def reboot(self):
	
		os.system("sudo reboot")
	
	# create a directory with a tag based on the datetime to store the .txt, inside the external memory
	def folder(self, ext):
	
		x = str(datetime.datetime.now())		# get the date and time
		x = x.split(" ")
		date = x[0].split("-")
		hours = x[1].split(":")
		tag = date[0] + date[1] + date[2] + "_" + hours[0] + hours[1]	# rearrange the date and time to create the tag
		self.path = "/media/pi/" + ext + "/" + tag + "_Arrays/"			# define the path
		try:
			os.mkdir(self.path)											# create the directory
		except OSError:													# handle the failure
			self.Inspector.folder("err")								# register the error in the LOG
			print("folder error")
			self.Alarm.delay(0.2)
			time.sleep(2)
			self.Alarm.pause()
			self.Alarm.delay(0.1)
		else:
			self.Inspector.folder("ok")									# register the correct operation in the LOG
		return 	tag	

	# create the file "index.txt" that will be used to convey information about the image set to the converter
	def index(self, wmax, width, height, bytes_per_pixel, missed):																								# creation of index with general information
		# the index is a json containing number of captures, frame dimensions and missed acquisition, to be skipped
		ipath = self.path + "Index.txt"
		index = {"n_samples":wmax+1,"width":width.value,"height":height.value,"bytes":bytes_per_pixel,"missed":missed}
		json_index = json.dumps(index)
		# try to create the file. Any error is signaled and registered in the LOG
		try:
			i = open(ipath,'w+')
			i.write(json_index)
			i.close()
		except IOError:
			self.Inspector.index("err")
			print("index error")
			self.Alarm.steadyON()
			time.sleep(2)
			self.Alarm.steadyOFF()
		else:
			self.Inspector.index("ok")

	# extract the number of layers from a .txt file in the external memory
	def parameters(self, ext):			# this will be erased once the start is done by trigger
	
		ppath = "/media/pi/" + ext + "/General.txt"
		
		# Open the file and read the json, register the error in the LOG, return the number of layers to the main.
		# if an error occurs it is returned 0, thus skipping the acquisition cycle
		try:
			g = open(ppath,"r")
			json_data = g.read()
			data = json.loads(json_data)
			g.close()
		except IOError:
			self.Inspector.parameters("err")
			print("parameter error")
			self.Alarm.delay(0.2)
			self.Alarm.enable()
			time.sleep(2)
			self.Alarm.pause()
			self.Alarm.delay(0.1)
			return 0
		else:
			self.Inspector.parameters("ok")
			layers = int(data["Layers"])
			return layers

	# write the LOG file with the information gained during all the different operations
	def log(self):

		self.Inspector.write(self.path)