import time
from pyueye import ueye
import sys
from SystemManager import *
from SLMCapture import *
from CircuitSet import *
from LOGManager import *
from SystemThreads import *

## Variables
hCam = ueye.HIDS(0)             			# 0: first available camera;  1-254: The camera with the specified camera ID -> visible/settable through software "IDS camera manager"
cInfo = ueye.CAMINFO()
pcImageMemory = ueye.c_mem_p()
MemID = ueye.int()
rectAOI = ueye.IS_RECT()				# default rectangular dimension
pitch = ueye.INT()
nBitsPerPixel = ueye.INT(8)
m_nColorMode = ueye.IS_CM_MONO8
bytes_per_pixel = int(nBitsPerPixel / 8)	

buttonpin = 12						# pin 12 = GPIO18
trigpin = 16						# pin 16 = GPIO23
LEDpin = 18							# pin 18 = GPIO24
startpin = 22						# pin 22 = GPIO25
ERRpin = 32							# pin 32 = GPIO12
EXPpin = 29							# pin 29 = GPIO5
start = 0							# starter trigger
flag = 0							# will be used to verify that the camera successfully took the shot
w = 0								# counter
errcounter = 0						# counter of missed acquisition, initialize the camera all over if it reaches a threshold
ext = "GIO"							# name of the external memory
missed = []

# create the objects
Inspector = LOGManager()									# handler of LOG
Butler = CircuitSet(buttonpin, startpin, trigpin)			# takes care of setting the board
Wink = BlinkThread(LEDpin, 0.1)								# status LED
Alarm = BlinkThread(ERRpin, 0.1)							# error LED for main operations
ExportErr = BlinkThread(EXPpin, 0.2)						# error LED for export operations
Doc = SystemManager(Alarm,Inspector)						# manage the system operations, such as files and fodlers creation, and reboot and poweroff
Eve = SLMCapture(hCam,cInfo,pcImageMemory,MemID,rectAOI,pitch,nBitsPerPixel,m_nColorMode,bytes_per_pixel)			# manage the camera
Chronos = TimerThread()										# chronometer, used to measure timeout
#Kairos = TimerThread(Butler,trigpin)						# chronometer, used to measure recoater movement (still bugged because the 'wait for edge' isn't suited for threads)

allright = [1,1,1,1,1]										# error array

# Camera initialization, return frame dimensions and error flags
(width,height,allright[0],allright[1],allright[2]) = Eve.Initialization()

# Starting of the threads
Wink.start()
Alarm.start()
ExportErr.start()
Chronos.start()
#Kairos.start()

# status LED on, signaling it's ready
Wink.steadyON()

# the script wait for the command to begin the acquisitions given by the button. This will become a wait_for_edge once the second photoresistor is implemented
# Butler.Trigger(startpin,0)		# uncomment and remove below once the starting trigger is in place
Butler.Trigger(buttonpin,0)

# status LED off, signaling that the start was recieved
Wink.steadyOFF()

time.sleep(1)

# retrieve the parameters from a .txt file in the external memory, containing the number of layers
layers = Doc.parameters(ext)		# this will be erased once the start is done by trigger
wmax = int(layers)*2 + 1			# this will be erased once the start is done by trigger

## Create the folder with date and time, for multiple acquisition. Takes in input the name of the external memory, return the tag based on date and time
tag = Doc.folder(ext)

Pablo = ExportThread(ExportErr,Inspector,tag)				# exporter of array to memory
Pablo.start()

allright[3] = Eve.Memalloc()
allright[4] = Eve.Queue()

## Update the LOG with the informations currently available
Inspector.register(tag)
Inspector.camera(allright)

## Error check in camera operation. A "0" in the array would mean the camera cannot properly work, thus going for a reboot
if 0 in allright:
	Alarm.steadyON()
	Butler.Trigger(buttonpin,0)
	Doc.log()
	Butler.CircuitCleanup()
	Doc.reboot()

# second while cycle, the script perform the acquisitions and store the data in a txt file. This will become a read of the startpin status once the second photoresistor is implemented
# while (Butler.Read(startpin)):	# uncomment and remove below once the starting trigger is in place
while True:

	# this allows to break from the cycle if more than ten minutes pass without any trigger, meaning the number of layer set might have been too high
	Chronos.enable(600)
#	Kairos.monitor()
	Butler.Trigger(trigpin,1)
#	Kairos.catch()
	delta = Chronos.safe()	# 1 if timeout expired

	if delta == 1:
		Alarm.delay(0.5)
		Alarm.enable()
		time.sleep(3)
		Alarm.pause()
		Alarm.delay(0.1)
		break

	Wink.enable()						# status blink when the trigger is detected
	
	time.sleep(1.5)

	print("acquisition 1")
	(array,flag1) = Eve.Acquisition()	# returns the array and a flag that signal if the acquisition was a success
	
	if (flag1 != ueye.IS_SUCCESS):
		missed.append(w)				# notes the missing acquisition to avoid a stop in the conversion
		Inspector.acquisition(w)		# notes the error in the LOG
		Wink.pause()
		Alarm.enable()
		errcounter = errcounter + 1		# increase the number of error in a row
	else:
		errcounter = 0					# the error counter is a safety measure against the stop of the camera, if it acquires again a frame the counter reset
	
#	Kairos.monitor()
#	Kairos.catch()

	array1 = array.copy()				# NEEDED, or it associate the camera memory with the result of the function, thus overwriting it at the next acquisition
	
	Pablo.newby(flag1, w, array1)		# a new array goes to export

	w = w + 1
	if (w == wmax-1):
		break
	
	Wink.pause()

	time.sleep(0.85)

	Alarm.pause()
	Wink.enable()				# the led blink since the recoater is moving, but it will wait until it stops to capture the powder bed
	
	time.sleep(2)

	print("acquisition 2")
	(array,flag2) = Eve.Acquisition()

	if (flag2 != ueye.IS_SUCCESS):
		missed.append(w)
		Inspector.acquisition(w)
		Wink.pause()
		Alarm.enable()
		errcounter = errcounter + 1
	else:
		errcounter = 0

	array2 = array.copy()

	Pablo.newby(flag2, w, array2)

	Wink.pause()
	time.sleep(1)
	Alarm.pause()

#	Doc.export(array1, w-1, flag1)
#	Doc.export(array2, w, flag2)
	
	w = w + 1
	if (w == wmax-1):
		break

	if errcounter >= 10:			# if there are more than tot failed acquisition in a row, the camera memory gets released and the initialization is repeated
		Eve.exit()
		time.sleep(2)
		(width,height,allright[0],allright[1],allright[2]) = Eve.Initialization()
		allright[3] = Eve.Memalloc()
		allright[4] = Eve.Queue()
		if 0 in allright:
			Alarm.steadyON()
			Butler.Trigger(buttonpin,0)
			Doc.log()
			Butler.CircuitCleanup()
			Doc.reboot()

		

# post acquisition, for comparison

time.sleep(5)
print("post acquisition")
(array_post,flag) = Eve.Acquisition()
flag3 = 0

if (flag != ueye.IS_SUCCESS):
	missed.append(w)
	Inspector.acquisition(w)
	flag3 = 1
	Alarm.enable()
	time.sleep(2)
	Alarm.pause()

Pablo.newby(flag3, w, array_post)

while auth == 0:
	auth = Pablo.authorization()

Doc.index(w, width, height, bytes_per_pixel, missed)

Eve.Exit()

Wink.delay(0.5)
Wink.enable()
time.sleep(3)

Wink.stop()
Alarm.stop()
ExportErr.stop()
Chronos.stop()
Pablo.stop()
#Kairos.stop()

Butler.CircuitCleanup()
Doc.log()

Doc.reboot()