from __future__ import division
from shapely.geometry import Polygon
from scipy import ndimage as ndi
from skimage import filters, segmentation, measure
import numpy as np
import matplotlib.pyplot as plt

class ActiveContouring():
	
	def __init__(self,OP):
		
		self.OP = OP
		
	def Extraction(self,image,expansion,ratio,minarea,minheigth):
		
		############################ INITIALIZATION ############################
		
		(x,y) = image.shape
		filled = self.OP.filled_edges(image,1.3,20)								# initial segmentation
		labeled, num = ndi.label(filled)										# labelling
		test = np.zeros_like(image)
		mask = np.zeros_like(image)
		contoured = np.zeros_like(image)
		unique,counts = np.unique(labeled,return_counts=True)					# creation of dictionary associating each object and its pixel count (area)
		val = dict(zip(unique,counts))

		for key in val:															# remove the objects below a certain threshold to filter noise
			if val[key] <= minarea:
				labeled[labeled == key] = 0
		
		########################## EXPANSION ###################################
		
		for iterator in range(expansion):										# expand the labeled image without merging with nearby objects
			sp = expansion + 3 - iterator										# multiple iteration start from large expansion the decrease, to minimize number of cycle
			cp = sp - 1															# while sp define the area checked for contact with other object, cp define the area to overwrite
			for i in range(x):
				for j in range(y):
					temp = labeled[i-sp:i+sp,j-sp:j+sp]							# the control neighbourhood is defined
					w,z = temp.shape
					flagtemp = 0												# flag used to signal the possibility of expansion: 0 means it can expand
					if labeled[i,j] != 0:										# check if the pixel belong to an object
						for k in range(w):
							for l in range(z):
								if labeled[i-sp+k,j-sp+l] != labeled[i,j] and labeled[i-sp+k,j-sp+l] != 0:		# check if in the neighbourhood contains another object
									flagtemp = 1
					else:														# the original pixel didn't belong to an object, necessary to avoid expanding the contour
						flagtemp = 1
					if i >= sp and i <= x-sp-1 and j >= sp and j <= y-sp-1:		# check that the object is sufficiently distant from the border
						if flagtemp == 0:
							test[i-cp:i+cp,j-cp:j+cp] = labeled[i,j]			# overwrite the prop image
			labeled = test.copy()												# update the labeled image for the next iteration
		
		################# CREATION CONTOURED ##############################

		gaus = filters.sobel(filters.gaussian(filled,3))						# create an image where the edges are highlighted
		contours = measure.find_contours(labeled,0.8)							# create a contouring of the expanded labeled: these will be the firsts snakes
		
		for contour in contours:
			contour = np.fliplr(contour)										# match the indeces with the image
			snake = segmentation.active_contour(gaus,contour,alpha=0.001,beta=1.2,gamma=0.001,w_edge=0,w_line=10)		# create the snake
			pol = Polygon(snake)												# create polygons from snake
			minx,miny,maxx,maxy = pol.bounds									# measure the dimensions
			locratio = (maxx-minx)/(maxy-miny)									# create a ration horiz/vert
			if locratio <= ratio or maxy-miny >= minheigth:						# exclude horizontal defects by checking the ratio combined with the heigth
				mask[(snake[:,1]).astype('int'),(snake[:,0]).astype('int')] = 1	# update the mask with "1" value if the pixel is part of snake
		
		mask = filters.gaussian(mask,sigma=1)									# use a gaussian to connect part of the snake disconnected due to the trasformation in pixel
		mask = ndi.binary_fill_holes(mask)										# filled the contours to create the objects mask
		contoured[mask == True] = 1												# convert "True/False" in "1/0" format
		
		return contoured