from Horizontal import *
from Waves import *
from Incandescences import *
from Holes import *
from Spattering import *
from ImNormalization import *
from ActiveContouring import *

class Detective():
	
	def __init__(self, OP, flagbckgnd, background):								# object creation from all the functions
		
		self.OP = OP
		self.Hor = Horizontal(OP)
		self.Spat = Spattering(OP)
		self.Hol = Holes(OP)
		self.Incan = Incandescences(OP)
		self.Wave = Waves(OP)
		self.ImNorm = ImNormalization(OP)
		self.AcCont = ActiveContouring(OP)
		self.flagbckgnd = flagbckgnd
		
		ran = 5																	# width of the levels dividing the image (low ran, high resolution)
		if flagbckgnd == 1:
			self.ImNorm.Mask(background,ran)									# the creation of the mask for normalization is performed once at the initialization of the object
		
	def normalization(self,image):
		
		new = self.ImNorm.Correction(image)										# normalization of the image, based on the mask already created
		return new
		
	def contouring(self,image):
		
		pass
		
	def active_contouring(self,image):

		expansion = 2															# number of iteration for the expansion
		ratio = 4.5																# ratio horizontal/vertical to exclude horizontal lines from object recognition
		minarea = 150															# minimum are for the object to exclude spattering
		minheigth = 10															# minimum heigth. In combination with "ratio" allows to exclude horizontal but not objects with high horizontal dimension
		
		contoured = self.AcCont.Extraction(image,expansion,ratio,minarea,minheigth)		# creation of the contouring mask: objects are marked as "1", background as "0"
		
		return contoured
		
	def horizontal_detection(self,rawimage,contoured):

		vgap=3																	# maximum vertical distance to be considered horizontal
		hgap=15																	# maximum horizontal gap to be considered part of the same line
		
		(num,lengths,quadrant) = self.Hor.Detection(rawimage,contoured,vgap,hgap)		# horizontal extraction
		features = {"number":num,"lengths":lengths,"quadrant":quadrant}			# dictionary creation with the information of interest
		
		return features
		
	def spattering_detection(self,rawimage,contoured):

		maxarea = 50								# maximum area of the defect to be detected
		minarea = 10								# minimum area of the defect to be detected, to exclude noise in the image
		
		if self.flagbckgnd == 1:
			lt = 115
		else:
			lt = 180
		
		(num,area,marea,vector,quadrant) = self.Spat.Detection(rawimage,contoured,maxarea,minarea,lt)			# spattering extraction
		features = {"number":num,"area":area,"medium_area":marea,"vector":vector,"quadrant":quadrant}			# dictionary creation with the information of interest

		return features
	
	def holes_detection(self,rawimage):

		if self.flagbckgnd == 1:
			lvl = 65								# threshold of intensity under which a defect is detected
		else:
			lvl = 130								# threshold without normalization
		maxarea = 100								# maximum area, over that is either an error in acquisition or an induced powder deficit at the end of the job
		minarea = 8
		
		(num,area,quadrant) = self.Hol.Detection(rawimage,lvl,maxarea,minarea)		# holes extraction
		features = {"number":num,"area":area,"quadrant":quadrant}			# dictionary creation with the information of interest
		
		return features
	
	def incandescence_detection(self,rawimage,contoured):

		if self.flagbckgnd == 1:
			lvl = 180								# threshold of intensity over which a defect is detected
		else:
			lvl = 240								# threshold without normalization
		minarea = 25								# minimum area, under that is not considered a defect
		maxarea = 200								# to exclude the laser in case the image is badly timed
		
		(num,area,quadrant) = self.Incan.Detection(rawimage,contoured,lvl,maxarea,minarea)		# incandescence extraction
		features = {"number":num,"area":area,"quadrant":quadrant}			# dictionary creation with the information of interest
		
		return features
		
	def waves_detection(self,rawimage):

		(num,start,width) = self.Wave.Detection(rawimage)					# waves extraction
		features = {"number":num,"start":start,"width":width}				# dictionary creation with the information of interest
		
		return features