from __future__ import division
import skimage
import numpy as np
import time
import matplotlib.pyplot as plt
from scipy import ndimage as ndi
from skimage import filters, measure, feature, io, data, color, segmentation, morphology
from shapely.geometry import Polygon, Point

class PostImage:
		
	def med_value(self, image):
		## return the medium brightness value of the image
		
		(x,y) = image.shape
		for i in range(x):
			for j in range(y):
				data = data + image[i,j]
		data = data/(x*y)
		return data
	
	def convolve2D(self, image, kernel):
		## convolution of an image and a kernel, based on the karnel it can produce a sharper image or a blurred one
		
		kernel = np.flipud(np.fliplr(kernel))
		output = np.zeros_like(image)
		image_padded = np.zeros((image.shape[0]+2,image.shape[1]+2))
		image_padded[1:-1,1:-1] = image
		for x in range(image.shape[1]):
			for y in range(image.shape[0]):
				output[y,x] = (kernel*image_padded[y:y+3,x:x+3]).sum()
		
		return output
	
	def filled_edges(self, image, sig, res):
		## find the edges, create an elevation map with the edges to attain a numerical value of the edges,
		## then fill the holes and convert the filled image from "True/False" to "1/0"
		
		(x,y) = image.shape
		edg = skimage.feature.canny(image, sigma=sig)	# find the edges in the image
		edmap = filters.sobel(edg)						# assign a numerical value and define the borders with a Sobel operator
		filled = np.zeros_like(image)
		
		v = int(x/res)									# define the area which will be filled, to avoid accidental contact between distant objects resulting in a grat portion of background incorrectly filled
		c = int(y/res)

		for e in range(v):
			for o in range(c):
				frag = edmap[e*res:e*res+res,o*res:o*res+res]							# fragment of elevation map taken into account
				filled[e*res:e*res+res,o*res:o*res+res] = ndi.binary_fill_holes(frag)	# filled fragment added to the final image
		out = np.zeros_like(image)
		
		out[filled == True] = 1															# conversion from "True/False" to "1/0" format
		out[filled == False] = 0
		
		return out
	
	def light_identification(self, lt, ht, image):
		## generate markers in point brighter then a treshold, then watershed the elevation map according
		## to markers, and label the objects. It requires the tresholds and the image.
		
		markers = np.zeros_like(image)
		markers[image < lt] = 1												# create a mask of points above the threshold
		markers[image > ht] = 2
		elevation_map = filters.sobel(image)								# find the borders in the image with a Sobel operator
		segmentation = morphology.watershed(elevation_map, markers)			# create the watershed image
		
		segmentation[segmentation == 1] = 0									# transform the image in "0/1" format
		segmentation[segmentation == 2] = 1

		labeled, num = ndi.label(segmentation)								# label the image to enumerate the objects

		return (num,segmentation,labeled)
	
	def dark_identification(self, lt, ht, image):

		markers = np.zeros_like(image)
		markers[image < lt] = 2												# create a mask of points below the threshold
		markers[image > ht] = 1
		elevation_map = filters.sobel(image)								# find the borders in the image with a Sobel operator
		segmentation = morphology.watershed(elevation_map, markers)			# create the watershed image
		
		segmentation[segmentation == 1] = 0									# transform the image in "0/1" format
		segmentation[segmentation == 2] = 1
		
		labeled, num = ndi.label(segmentation)								# label the image to enumerate the objects
		
		return (num,segmentation,labeled)
	
	def centers_identification(self,labeled):
		
		(x,y) = labeled.shape
		labeled[0,:] = 0													# exclude the borders to avoid the contours spiking outisde the image
		labeled[x-1,:] = 0
		labeled[:,0] = 0
		labeled[:,y-1] = 0
		contours = measure.find_contours(labeled,0.8)						# identify contours in the input image
		centers = {}
		for contour in contours:											# swith colums in contour to match the format of the image
			temp = np.copy(contour[:,1])
			contour[:,1] = contour[:,0]
			contour[:,0] = temp
		for contour in contours:
			obj = Polygon(contour)											# create a polygon for each contour
			c = obj.centroid.coords[0]										# find center of the polygon: (horizontal, vertical), starting from top-left corner
			cx = int(c[1])
			cy = int(c[0])
			
			try:
				centers[str(labeled[cx,cy])] = (cx,cy)						# assign to each object the coordinates of the center: the numbers derive from the labelling of the objects
			except IndexError:
				pass
		
		return centers
	
	def quadrant_identification(self,centers,image):
		
		(x,y) = image.shape
		quadrants = {}
		for c in centers:													# extract centers
			cx = int(centers[c][0])
			cy = int(centers[c][1])
																			# assign the center to a quadrant depending on the coordinates. this configuration is meant for a 3x4 division
			if cx >= 0 and cx <= x/3:
				qx = 1
			if cx >= (x/3) and cx <= 2*x/3:
				qx = 2
			if cx >= (2*x/3) and cx <= x:
				qx = 3
			if cy >= 0 and cy <= y/4:
				qy = 1
			if cy >= (y/4) and cy <= y/2:
				qy = 2
			if cy >= (y/2) and cy <= 3*y/4:
				qy = 3
			if cy >= (3*y/4) and cy <= y:
				qy = 4

			try:
				quadrants[c] = (4*(qx-1)+qy)								# quadrant are numbered from left to right, from top to bottom
			except UnboundLocalError:
				pass
		return quadrants