from skimage import transform
import numpy as np
from shapely.geometry import Polygon, Point
import matplotlib.pyplot as plt

class Horizontal():

	def __init__(self,OP):
	
		self.OP = OP
	
	def Detection(self,image,contoured,vgap,hgap,res=15):

		filled = self.OP.filled_edges(image,0.92,50)							# segment the image
		
		values = [80,120,200]													# range of different lengths investigated
		final = []
		
		for value in values:
			
			lines = transform.probabilistic_hough_line(filled, threshold=10, line_length=value)		# PPHT: identify lines in the image with the reference length value, return the verices of the lines

			death = []
			llines = []
			
			# create a deathlist containing lines that are not horizontal
			for a,b in lines:
				if abs(a[1]-b[1]) >= vgap:
					death.append((a,b))
				else:															# create a list of points inside the extremes found by PPHT (lines)
					lline = np.linspace(a,b,50)
					llines.append(lline)

			for lline in llines:
				inside = 0
				start = lline[0]
				stop = lline[-1]
				for point in lline:
					a = int(point[0])
					b = int(point[1])
					if contoured[b,a] == 1:										# check if a specific point of the line is inside an object
						inside = inside + 1
				p,q,r,s = int(start[0]),int(start[1]),int(stop[0]),int(stop[1])	# convert the line in the format given by PPHT
				line = ((p,q),(r,s))
				if inside >= 5:													# if more than 4 points of a line are inside an object that line is listed to be removed
					death.append(line)

			# remove the lines in the list
			for el in death:
				if el in lines:
					lines.remove(el)

			# append the remaining lines in the final pool, that will be composed of the valid lines from each different length value
			for line in lines:
				final.append(line)

			# check if the lines point the same defect (function defined below)
			if len(final) > 0:
				final = self.Merging(final,values)

		n_lines = len(final)													# number of lines in the list
		length = []
		centers = {}
		w = 0
		
		# compute the centers of the lines and the quadrants in which the lines are by using the designed function
		if len(final) != 0:
			for t,u in final:
				w = w + 1
				length.append(abs(u[0]-t[0]))
				centers[w] = ((u[1]+t[1])/2,(u[0]+t[0])/2)
			quadrants = self.OP.quadrant_identification(centers,image)
		else:
			quadrants = {}
		
		return (n_lines,length,quadrants)
	
	
	def Merging(self,lines,samples):
		
		
		for k in range(len(samples)):  											# the function is repeated a number of times to 
			lines2 = []
			for line in lines:													# create a copy of the lines
				lines2.append(line)

			death2 = []
			for i in range(len(lines)):
				p = lines[i][0]
				q = lines[i][1]
				for r,s in lines2:												# if the lines are not the same, and the vertical coordinate is below 10, the lines are comparable
					if (p,q) != (r,s) and abs(p[1]-r[1]) <= 10:
						flagr,flags = 0,0
						pol = Polygon([(p[0],p[1]-5),(p[0],p[1]+5),(q[0],q[1]+5),(q[0],q[1]-5)])	# create a polygon in the neighbourhood of one of the lines
						pointr = Point(r[0],r[1])													# create points from the vertices of the second line in the comparison
						points = Point(s[0],s[1])
						if pol.contains(pointr):													# check if the points are inside the polygon
							flagr = 1
						if pol.contains(points):
							flags = 1
						# depending if only one point is inside, both or none, different actions are required: if none, simply skip, if both delete the line completely inside, if only one point the two lines
						# need to be merged, creating a new line with the point to the far left and the point to the far right
						mult,add = flagr*flags,flags+flagr
						if mult:												# both flags are 1, one line is entirely inside the other
							death2.append((r,s))
						if add == 1:
							if flagr == 1:										# one point is inside, the other is outside. the outside point will be combined with the farthest point to create a new line
								if abs(s[0]-p[0]) <= abs(s[0]-q[0]):
									lines[i] = (s,q)
								else:
									lines[i] = (p,s)
							if flags == 1:
								if abs(r[0]-p[0]) <= abs(r[0]-q[0]):
									lines[i] = (r,q)
								else:
									lines[i] = (p,r)
			# remove lines completely inside
			for el in death2:
				if el in lines:
					lines.remove(el)
	
		return lines