from Detective import *
from Functions import *
from skimage import io
import json
import os

ymin,ymax,xmin,xmax = 200,900,250,1050																	# crop the images depending on the camera inclination
features = {}

curr = os.getcwd()																						# get current directory

# look for the images folder and create the destination file: in this configuration, the images folder MUST have "Display" in the name
for filename in os.listdir(curr):																		
	if 'Display' in filename:
		src = filename.split("_")
		dst = src[0] + "_" + src[1] + "_Features.txt"
		break

folder = curr + "/" + filename																			# folder of images
destination = curr + "/" + dst																			# destination file (maintain the timestamp, add "Features.txt"

# index reading, if any other way is used to convey information on the process, edit here.
# MACHINE PARAMETERS WILL BE EXTRACTED HERE WHEN PRESENT
i = open(folder + "/Index.txt","r")
index = json.loads(i.read())
wmax = index["n_samples"]																				# extract number of images
missed = index["missed"]																				# extract list of missed acquisition
success = index["success"]																				# extract result of the job
parameters = index["parameters"]																		# extract parameters information
i.close()

# check the existence of a background image for normalization, saved in the acquisition with "post_image" name
for image in os.listdir(folder):
	if 'post' in image:
		background = io.imread(folder + "/" + image)[ymin:ymax,xmin:xmax]
		flagbckgnd = 1																					# indicate the presence of a background image
		break
	else:
		background = None
		flagbckgnd = 0
# create the objects for the feature extraction
OP = PostImage()
Conan = Detective(OP,flagbckgnd,background)

w = 0																									# initialize counter
while w <= wmax:
	
	if w%2 == 0:																						# register the new layer every two images
		n_layer = "layer " + str(int(w/2)+1)
		layer = {}

	if w not in missed:																					# skip the missed acquisitions
		raw = io.imread(folder + "/Image" + str(w) + ".jpg")[ymin:ymax,xmin:xmax]						# crop the image
		if flagbckgnd == 1:
			image = Conan.normalization(raw)															# normalize the image if a background is present
		else:
			image = raw																					# use the unedited image if there's no background
			
		if w % 2 == 0:																					# an even number means the image contains a complete layer
			#print("image: " + str(w))
			waves = Conan.waves_detection(image)														# extract waves
			layer["waves"] = [waves]
			contoured = Conan.active_contouring(image)													# return a mask with value "1" where an object is identified, "0" for background
			hor = Conan.horizontal_detection(image,contoured)											# extract horizontal
			layer["horizontal"] = [hor]
			spat = Conan.spattering_detection(image,contoured)											# extract spattering
			layer["spattering"] = [spat]
			incandescence = Conan.incandescence_detection(image,contoured)								# extract incandescence
			layer["incandescence"] = [incandescence]
		else:																							# an odd number means the image contains a new layer
			hol = Conan.holes_detection(image)															# extract holes
			layer["holes"] = [hol]

	features[str(n_layer)] = layer																		# create the defect information for the current layer
	w = w + 1

features["parameters"] = parameters																		# pass the parameters information
features["success"] = success																			# pass the result information

# save the information on a .txt file
json_features = json.dumps(features)
g = open(destination,"w+")
g.write(json_features)
g.close()