from __future__ import division
import math
import numpy as np
from sklearn import cluster
from scipy import ndimage as ndi
import matplotlib.pyplot as plt

class Spattering():

	def __init__(self,OP):
		
		self.OP = OP
		
	def Detection(self,raw_image,contoured,maxarea,minarea,lt):

		################# OBJECT IDENTIFICATION AND EXCLUSION #####################
		
		labobj,objnum = ndi.label(contoured)										# objects are enumerated
		obj_centers = self.OP.centers_identification(labobj)						# centers of the objects are identified (will be used in vector creation)
		objects = obj_centers.values()
		
		###########################################################################
		###################### SPATTERING IDENTIFICATION ##########################
		
		filled = self.OP.filled_edges(raw_image,1,30)								# create a segmented image
		filled[contoured == 1] = 0													# exclude points belonging to objects
		filled[raw_image >= lt] = 0													# exclude brighted areas than a defined threshold
		(num, segmentation, labeled) = self.OP.light_identification(1, 0, filled)	# enumerate the remainig "objects"

		unique,counts = np.unique(labeled,return_counts=True)						# create a dictionary in which to each number is associated the number of pixels with that value: area measuring
		val = dict(zip(unique,counts))

		centers = self.OP.centers_identification(labeled)
		quadrants = self.OP.quadrant_identification(centers,labeled)

		val[0] = 0
		
		###########################################################################
		########################## SPATTERING FILTERING ###########################		
		
		num = num + 1																# enumaration start with "0", so the highest number isn't counted, while the background will surely be excluded
		death = []
		areatot = 0
		w = 0
		
		# exclude the objects outside the range of accettable area, by registering the number of the object in a deathlist
		for key in val:
			if val[key] <= minarea or val[key] >= maxarea:
				if key not in death:
					death.append(key)
					num = num - 1

		# remove element in the deathlist from dictionary of values, centers and quadrants: they all follow the same enumeration
		for el in death:
			if el in val:
				del val[el]
			if str(el) in centers:
				del centers[str(el)]
			if str(el) in quadrants:
				del quadrants[str(el)]
				
		# add the remaining areas to compute total area
		for key in val:
			areatot = areatot + val[key]
			w = w + 1

		# compute medium area
		try:
			medarea = areatot/w
		except ZeroDivisionError:
			medarea = 0
	
		###########################################################################
		######################## VECTOR ASSOCIATION ###############################

		coords = centers.values()
		vectors = []
		numobj = objnum
		
		# define the number of objects, kepping a maximum value of 4: it is observed that it easily encounter errors above that value
		if objnum != 0 and len(coords) != 0:
			if objnum >= len(coords):
				numobj = len(coords)
			if numobj >= 5:
				numobj = 4

			kmeans = cluster.KMeans(n_clusters=numobj,random_state=0).fit(coords)		# kmeans clustering grouping the identified spattering
			points = kmeans.cluster_centers_											# points extraction

			# association of points just found with the nearest object's center
			for point in points:
				dist0 = 1000
				for obj in objects:
					dist = math.sqrt((point[0]-obj[0])**2+(point[1]-obj[1])**2)
					if dist <= dist0:
						dist0 = dist
						vector = [(obj[0],obj[1]),(point[0],point[1])]
				if vector not in vectors:
					vectors.append(vector)

		##########################################################################
		
		return (num,areatot,medarea,vectors,quadrants)