from __future__ import division
import numpy as np
import math
from sklearn.cluster import DBSCAN
from skimage import transform
import matplotlib.pyplot as plt

class Waves():
	
	def __init__(self,OP):
		
		self.OP = OP
		
	def Detection(self,rawimage):

		various = [400,550,650]											# multiple lengths are used	

		filled = self.OP.filled_edges(rawimage,0.9,20)					# the image is segmented
		deathlist = []
		lines = []
		widths = []
		bottom = []
		
		# lines identification with range created depending on y-coord
		defrange = 10			# indicate how much the angle changes in the frame
		startangle = 85			# used as a reference for the starting angle
		
		for value in various:
			line = transform.probabilistic_hough_line(filled,threshold=15,line_length=value,line_gap=20)		# PPHT used to find lines in the image with reference length
			for p,q in line:
				try:
					locangle = math.atan((p[1]-q[1])/(q[0]-p[0]))*180/math.pi									# compute the angle of the line
					if locangle <= 0:
						locangle = 180 + locangle
				except ZeroDivisionError:
					locangle = 90
				z = startangle + defrange*q[0]/800						# z is the possible angle for that coordinate
				feas = np.linspace(z-2,z+2,1000)						# a tolerance is added to create the feasible range
				for i in range(len(feas)):								# check if the line's inclination is in the range, else create a deathlist
					feas[i] = round(feas[i],2)
				if round(locangle,2) not in feas:
					deathlist.append((p,q))
			
			# remove lines in deathlist, then the remaining lines will be added to the total group (from each different length)
			for p,q in deathlist:
				if (p,q) in line:
					line.remove((p,q))
			for p,q in line:
				lines.append((p,q))

		# create a list of bottom coordinates to find the starting point (farther left) and the range covered by the lines
		for u,v in lines:
			widths.append(u[0])
			widths.append(v[0])
			bottom.append([u[0],u[1]])
				
		# group the lines by proximity using a dbscan clustering, with reference distance 100
		if len(bottom) != 0:
			db = DBSCAN(eps=100).fit(bottom)
			labels = db.labels_
			num = max(labels)+1											# number of clusters
		else:
			num = 0
			
		# compute the width covered and the starting point
		if num > 0:
			try:
				width = max(widths) - min(widths)
				start = max(widths)
			except ValueError:
				width = 0
				start = 0
		else:
			width = 0
			start = 0
		
		
		return (num,start,width)