import numpy as np
import matplotlib.pyplot as plt

class ImNormalization():
	
	def __init__(self, OP):
		
		self.mask = None
		self.final = []																		# list with the mean values of the different levels
		self.OP = OP
		
	def Mask(self,bkgnd,n):																	# creation of the mask from the background image, once for the cycle
		
		clusters = np.zeros_like(bkgnd)
		(x,y) = clusters.shape
		ref = []
		num = 255/n - ((255%n)/n) + 2														# number of levels

		for a in range(num):
			ref.append(a*n)																	# limits for the levels (multiples of n)

		for b in range(num-1):																# "b" is the number of level
			med = 0
			den = 0
			for i in range(x):
				for j in range(y):
					if bkgnd[i,j] >= ref[b] and bkgnd[i,j] <= ref[b+1]:
						clusters[i,j] = b													# create the "level" image, linking levels and coordinates
						med = med + bkgnd[i,j]												# save the total values and the number of pixels to compute the mean value
						den = den + 1
			try:
				lvlmed = med/den
			except ZeroDivisionError:
				lvlmed = 0
			self.final.append(lvlmed)														# list of mean value for each level, exploiting the fact that element of a list are fixed in position
			self.mask = clusters															# association between coordinates and level
			
	def Correction(self,image):
		
		if self.mask.any() != None:
			new = np.zeros_like(image)														# normalized image
			sigma = 1																		# variable for segmentation (low sigma, lot of noise)
			frame = 15																		# frame for the filling (low frame, high fidelty to borders)
			(x,y) = image.shape
			
			filled = self.OP.filled_edges(image,sigma,frame)								# segment the image to identify the objects

			immed = {}
			imden = {}
			imfinals = {}
			deltas = {}

			for c in range(len(self.final)):												# initialization of dictionaries containing total sum of values and number of pixels
				immed[c] = 0
				imden[c] = 0

			for i in range(x):																# total value and number of pixels are noted, dividing the image by coordinates of to the levels saved in "mask"
				for j in range(y):
					if filled[i,j] == 0:													# exclude the pixels belongin to an object
						immed[self.mask[i,j]] = immed[self.mask[i,j]] + image[i,j]
						imden[self.mask[i,j]] = imden[self.mask[i,j]] + 1

			for c in range(len(self.final)):												# compute the deltas between the medium values of image and background for each level
				try:
					imfinal = immed[c]/imden[c]
				except ZeroDivisionError:
					imfinal = 0
				imfinals[c] = imfinal
				delta = imfinal-self.final[c]
				deltas[c] = delta

			for i in range(x):																# remove the delta from the images pixels, regardless of them belonging to an object or not
				for j in range(y):
					new[i,j] = image[i,j] - deltas[self.mask[i,j]]
					if new[i,j] >= 255:														# limitations for 8bits
						new[i,j] = 255
					if new[i,j] <= 0:
						new[i,j] = 0

		return new